package rogue 
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFormat;
	
	import com.greensock.TweenLite;
	
	/**
	 * ...
	 * @author Arthur Zeferino
	 */
	public class HUD extends Sprite
	{
		
		private var lifebarPlayer:Lifebar;
		private var lifebarEnemies:Array;
		
		private var txtVS:TextField;
		
		private var txtWin:TextField;
		private var txtLose:TextField;
		private var txtLose2:TextField;
		private var txtRound:TextField;
		
		private var playAgainButton:Sprite;
		
		public function HUD() 
		{
			lifebarPlayer = new Lifebar("PLAYER");
			lifebarPlayer.x = 5;
			lifebarPlayer.y = 5;
			
			this.addChild(lifebarPlayer);
			
			lifebarEnemies = new Array();
			
			txtVS = new TextField();
			txtVS.selectable = false;
			txtVS.embedFonts = true;
			txtVS.width = 80;
			txtVS.defaultTextFormat = new TextFormat("Vectrexfake", 90, 0xff0000);
			txtVS.text = "VS";
			
			txtVS.x = 280;
			txtVS.y = 15;		
			
			this.addChild(txtVS);
			
			txtWin = new TextField();
			txtWin.selectable = false;
			txtWin.embedFonts = true;
			txtWin.width = 600;
			txtWin.defaultTextFormat = new TextFormat("Vectrexfake", 100, 0xff0000);
			txtWin.text = "VICTORY";
			txtWin.alpha = 0;
			
			txtWin.x = 200;
			txtWin.y = 350;	
			
			txtLose = new TextField();
			txtLose.selectable = false;
			txtLose.embedFonts = true;
			txtLose.width = 600;
			txtLose.defaultTextFormat = new TextFormat("Vectrexfake", 100, 0xff0000);
			txtLose.text = "GAME OVER";
			txtLose.alpha = 0;
			
			txtLose.x = 160;
			txtLose.y = 200;					
		}
		
		public function updateLife(id:int, amount:Number):void
		{
			if (id == 0)
			{
				lifebarPlayer.setLife(amount);
			}
			else
			{
				id--;
				lifebarEnemies[id].setLife(amount);
			}
		}
		
		public function addEnemyLifeBar(name:String):void
		{
			var life:Lifebar = new Lifebar(name);
			
			life.x = 420;
			life.y = 5+(lifebarEnemies.length * 42);
			
			lifebarEnemies.push(life);
			
			addChild(life);
		}
		
		public function removeEnemyLifeBar(name:String):void
		{
			var i:int = 0;
			for (i = 0; i < lifebarEnemies.length; i++)
			{
				if (lifebarEnemies[i].playerName.text == name)
				{
					this.removeChild(lifebarEnemies[i]);
					break;
				}
			}
			
			lifebarEnemies.splice(i, 1);
			
			for (i = 0; i < lifebarEnemies.length; i++)
			{
				lifebarEnemies[i].x = 380;
				lifebarEnemies[i].y = 5+(i * 42);
			}
		}
		
		public function showRoundStart(round:int):void
		{
			txtRound = new TextField();
			txtRound.selectable = false;
			txtRound.embedFonts = true;
			txtRound.width = 600;
			txtRound.defaultTextFormat = new TextFormat("Vectrexfake", 100, 0xff0000);
			txtRound.text = "ROUND "+toRoman(round);
			txtRound.alpha = 0;
			
			txtRound.x = 200;
			txtRound.y = 350;
			
			this.addChild(txtRound);
			
			TweenLite.to(txtRound, 0.1, { alpha:1.0, delay:2.0, onComplete:roundDisplayed } );
		}
		
		public function roundDisplayed():void
		{
			TweenLite.to(txtRound, 0.1, { alpha:0.0, delay:2.0, onComplete:removeRoundDisplay } );
			this.dispatchEvent(new Event("ROUND_DISPLAYED"));
		}
		
		public function removeRoundDisplay():void
		{
			this.removeChild(txtRound);
			this.dispatchEvent(new Event("ROUND_STARTED"));
		}	
		
		public function displayWin():void
		{
			txtWin.alpha = 0;
			
			this.addChild(txtWin);
			
			TweenLite.to(txtWin, 0.1, { alpha:1.0, delay:2.0, onComplete:winDisplayed } );
		}
		
		public function winDisplayed():void
		{
			TweenLite.to(txtWin, 0.1, { alpha:0.0, delay:3.0, onComplete:removeWinDisplayed } );
			SoundManager.getInstance().playSound(GlobalSounds.SFX_WIN);
			SoundManager.getInstance().playSound(GlobalSounds.SFX_PUBLIC);
		}		
		
		public function removeWinDisplayed():void
		{
			this.removeChild(txtWin);
			this.dispatchEvent(new Event("WIN_DISPLAYED"));
		}
		
		public function showGameOver(round:int):void
		{
			txtLose2 = new TextField();
			txtLose2.selectable = false;
			txtLose2.embedFonts = true;
			txtLose2.width = 600;
			txtLose2.defaultTextFormat = new TextFormat("Vectrexfake", 80, 0xff0000);
			txtLose2.text = "YOU DEFEATED "+toRoman(round-1)+" ROUNDS";
			txtLose2.alpha = 0;
			
			txtLose2.x = 50;
			txtLose2.y = 300;
			
			this.addChild(txtLose);
			this.addChild(txtLose2);
			TweenLite.to(txtLose, 0.1, { alpha:1.0, delay:2.0, onComplete:loseDisplayed } );			
			TweenLite.to(txtLose2, 0.1, { alpha:1.0, delay:2.0} );			
		}
		
		public function loseDisplayed():void
		{
			playAgainButton = new Sprite();
			playAgainButton.buttonMode = true;
			
			var readyFill:Sprite = new Sprite();
			
			readyFill.graphics.lineStyle(2, 0xff0000);
			readyFill.graphics.beginFill(0x000000);
			readyFill.graphics.drawRect(0, 0, 150, 50);
			readyFill.graphics.endFill();
			
			playAgainButton.addChild(readyFill);
			
			var readyText:TextField = new TextField();
			readyText.width = 200;
			readyText.selectable = false;
			readyText.embedFonts = true;
			readyText.defaultTextFormat = new TextFormat("Vectrexfake", 50, 0xff0000);
			readyText.text = "PLAY AGAIN";
			playAgainButton.addChild(readyText);
			
			playAgainButton.x = 220;
			playAgainButton.y = 600;
			
			this.addChild(playAgainButton);
			
			playAgainButton.addEventListener(MouseEvent.MOUSE_UP, onPlayAgain);
		}
		
		public function onPlayAgain(event:MouseEvent):void
		{
			SceneManager.getInstance().setScene(new MenuScreen());
		}
		
		public function toRoman(n:int):String
		{
			if (n == 0)
				return "O";
			
			var num:String = "";
			
			if (n > 10)
			{
				var dec:int = n / 10;
				n = n % 10;
				
				for (var i:int = 0; i < dec; i++)
					num += "X";
			}
			
			num += getRomanDec(n);
			
			return num;
		}
			
		public function getRomanDec(n:int):String
		{
			switch(n)
			{
				case 1:
					return "I";
				case 2:
					return "II";
				case 3:
					return "III";
				case 4:
					return "IV";
				case 5:
					return "V";
				case 6:
					return "VI";
				case 7:
					return "VII";
				case 8:
					return "VIII";
				case 9:
					return "VIX";
			}
			
			return "";
		}
	}

}