package rogue 
{
	import flash.display.Sprite;
	import com.greensock.TweenLite;
	import com.greensock.easing.*;
	/**
	 * ...
	 * @author Arthur Zeferino
	 */
	 
	public class Hero extends GameSprite
	{
		private var sword:Sprite;
		private var body:Sprite;
		private var shield:Sprite;
		private var armor:Sprite;
		
		private var strength:Number;
		
		private var attacking:Boolean = false;
		private var holdingAttack:Boolean = false;
		
		private var hp:int = 100;
		
		public function Hero() 
		{
			body = new Sprite();
			body.graphics.lineStyle(3, 0x0000ff);
			body.graphics.drawCircle(20, 20, 20);
			body.x = -20;
			body.y = -20;
			this.addChild(body);
			
			sword = new Sprite();
			this.addChild(sword);
			sword.x = 5;
			sword.y = 20;
			
			sword.graphics.lineStyle(3, 0xffffff);
			sword.graphics.lineTo(30, 0);
			sword.rotation = 10;
			
			shield = new Sprite();
			this.addChild(shield);
			shield.x = 20;
			shield.y = -15;
			
			shield.graphics.lineStyle(3, 0xffffff);
			shield.graphics.lineTo(25, 0);
			shield.rotation = 70;		
			
			this.collider = body;
		}
		
		public function prepareAttack():void
		{
			if (!holdingAttack)
			{
				holdingAttack = true;
				strength = 0;
				sword.x = 40;
				sword.rotation = 0;
			}
			else
			{
				if (strength < 70)
				{
					strength+=10;
				}
				else
				{
					if (strength == 71)
						strength -= 1;
					else
						strength += 1;
				}
				
				sword.rotation = strength+30;
				var rad:Number = (strength+30) * Math.PI / 180;
				sword.x = (24 * Math.cos(rad));
				sword.y = (24 * Math.sin(rad));
			}
		}
		
		/*Attack Mechanics*/
		public function attack():void
		{
			TweenLite.to(sword, strength/200, { rotation:-strength, onUpdate:updateSwordPos, onComplete:haltSword, ease:Quad.easeIn});
			holdingAttack = false;
			attacking = true;
		}
		
		protected function updateSwordPos():void
		{
			var rad:Number = (sword.rotation) * Math.PI / 180;
			sword.x = (24 * Math.cos(rad));
			sword.y = (24 * Math.sin(rad));
		}
		
		protected function haltSword():void
		{
			strength = 0;
			sword.x = 5;
			sword.y = 20;
			sword.rotation = 10;
			attacking = false;
		}
		
		public function isAttacking():Boolean
		{
			return attacking;
		}
		
		public function isHoldingAttacking():Boolean
		{
			return holdingAttack;
		}
		
		/*Shield Mechanics*/
		public function shieldUp():void
		{
			shield.x = 25;
			shield.y = -5;
			shield.rotation = 90;
		}
		
		public function shieldDown():void
		{
			shield.x = 20;
			shield.y = -15;
			shield.rotation = 70;
		}		
		
		public function isDefending():Boolean
		{
			return shield.x > 20;
		}
		
		public function get weaponSprite():Sprite
		{
			return sword;
		}
		
		public function get shieldSprite():Sprite
		{
			return shield;
		}
		
		public function updateRotation(mX:Number, mY:Number):void
		{
			this.rotation = (Math.atan2(mY-y, mX-x) * 180 / Math.PI) - 30;
		}
		
		
		public function getStrength():Number
		{
			return strength;
		}
		
		override public function update():void
		{
			
		}
	}

}