package rogue 
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.geom.Point;
	import flash.ui.Keyboard;
	import com.greensock.TweenLite;
	import com.greensock.easing.BackOut;
	
	/**
	 * ...
	 * @author Arthur Zeferino
	 */
	public class MainGame extends GameScene
	{		
		private var crowd:CrowdSpt;
		
		private var player:Warrior;
		private var map:LevelMap;
		
		private var enemies:Array;
		
		private var hud:HUD;
		
		private var currentRound:int = 1;
		private var gameStarted:Boolean = false;
		
		public function MainGame() 
		{
			super();			
			
			map = new LevelMap();
			addChild(map);
			
			player = new Warrior();
			
			player.initialize(GlobalPlayerData.playerWeapon, GlobalPlayerData.playerHat, 0xffff00);
			
			this.addChild(player);
			player.x = 200;
			player.y = 384;	
			
			enemies = new Array();
			
			gameStarted = false;
			
			crowd = new CrowdSpt();
			crowd.y = 600;
			this.addChild(crowd);
			
			hud = new HUD();
			this.addChild(hud);			
			
			initialize();
		}
		
		public function initialize():void
		{
			hud.addEventListener("ROUND_DISPLAYED", onRoundPrepare);
			hud.addEventListener("ROUND_STARTED", onRoundStart);
			hud.addEventListener("WIN_DISPLAYED", onPlayerWin);
			startRound();
		}
		
		public function startRound():void
		{
			SoundManager.getInstance().setMusicVolume(0.1);
			hud.showRoundStart(currentRound);
		}
		
		public function onRoundStart(event:Event):void
		{
			gameStarted = true;
		}
		
		public function onRoundPrepare(event:Event):void
		{
			var warriorNames:Array = new Array();
			for each (var name:String in Enemy.WarriorNames)
			{
				warriorNames.push(name);
			}
			
			var maxWarriors:int = (currentRound < 5) ? currentRound : 4;
			var stronger:int = currentRound % 4;
			var minDif:int = currentRound / 10;
			
			for (var i:int = 0; i < maxWarriors; i++)
			{
				var enemy:Enemy = new Enemy();
				enemy.initialize(Math.floor(Math.random()*3), Math.floor(Math.random()*4), 0x00ff00);
				enemy.x = 325+Math.cos(-(Math.PI/maxWarriors)*i)*280;
				enemy.y = 410+Math.sin(-(Math.PI/maxWarriors)*i)*280;
				enemy.rotation = 120;
				this.addChild(enemy);
				
				enemy.gamePtr = this;
				if (stronger > 0)
				{
					enemy.setDificulty(minDif + 1);
					stronger--;
				}
				else
					enemy.setDificulty(minDif);
				
				enemy.setEnemyArray(enemies);
				
				enemies.push(enemy);
				
				enemy.warriorName = warriorNames.splice(Math.floor(Math.random() * warriorNames.length), 1)[0];
				
				this.setChildIndex(enemy, 0);
				
				hud.addEnemyLifeBar(enemy.warriorName);
			}
			
			SoundManager.getInstance().playSound(GlobalSounds.SFX_PUBLIC);
			crowd.animate(0.5);
		}
		
		override public function update():void
		{
			
			var playerPos:Point = new Point(player.x, player.y);
			
			if (!player.controlLost && player.active)
			{
				if (Input.isPressed(Keyboard.A))
				{
					player.x -= 10;
				}
				else if (Input.isPressed(Keyboard.D))
				{
					player.x += 10;
				}
				
				if (Input.isPressed(Keyboard.W))
				{
					player.y -= 10;
				}
				else if (Input.isPressed(Keyboard.S))
				{
					player.y += 10;
				}
				
				if (Input.isMousePressed())
				{
					player.prepareAttack();
				}
				else if (player.isHoldingAttacking())
				{
					player.attack();
				}
				
				if (Input.isPressed(Keyboard.SHIFT))
				{
					player.shieldUp();
				}
				else if (player.isDefending())
				{
					player.shieldDown();
				}
				
				player.updateRotation(mouseX, mouseY);
			}
			
			/*if (map.checkCollision(player))
			{
				player.x = playerPos.x;
				player.y = playerPos.y;
			}*/
			
			crowd.update();
			
			if (gameStarted && player.active)
				checkEnemyCollisionAndUpdate();
		}
		
		public function checkEnemyCollisionAndUpdate():void
		{
			var activeEnemies:int = 0;
			
			var toRemove:Array = new Array();
			
			var soundChance:int;
			
			for (var i:int = 0; i < enemies.length; i++)
			{
				if (enemies[i].active)
				{
					if (player.hitTestObject(enemies[i]))
					{
						if (player.isAttacking())
						{
							if (player.weaponSprite.hitTestObject(enemies[i].collider))
							{
								if (enemies[i].isDefending() && (player.rotation*enemies[i].rotation < 0))
								{
									player.openDefense();
									enemies[i].reactDefense(player.getStrength());
								}
								else
								{
									enemies[i].hurt(player.getStrength());
									hud.updateLife(i + 1, enemies[i].hp / enemies[i].maxHp);
									
									soundChance = Math.floor(Math.random() * 200);
									if (soundChance < 100)
									{
										SoundManager.getInstance().playSound(GlobalSounds.SFX_PUBLIC1);
										crowd.animate(0.2);
									}
									else if (soundChance < 50)
									{
										SoundManager.getInstance().playSound(GlobalSounds.SFX_PUBLIC2);
										crowd.animate(0.2);
									}
								}
								
							}

						}
						else if (enemies[i].isAttacking())
						{
							if (enemies[i].weaponSprite.hitTestObject(player.collider))
							{
								if (player.isDefending() && (player.rotation*enemies[i].rotation < 0))
								{
									enemies[i].openDefense();
									player.reactDefense(enemies[i].getStrength());
								}
								else
								{
									player.hurt(enemies[i].getStrength());
									hud.updateLife(0, player.hp / player.maxHp);
									
									soundChance = Math.floor(Math.random() * 200);
									if (soundChance < 100)
									{
										SoundManager.getInstance().playSound(GlobalSounds.SFX_PUBLIC1);
										crowd.animate(0.2);
									}
									else if (soundChance < 50)
									{
										SoundManager.getInstance().playSound(GlobalSounds.SFX_PUBLIC2);
										crowd.animate(0.2);
									}
								}
								
							}

						}
					}
					
					
					enemies[i].update();
					activeEnemies++;
				}
				else
				{
					this.setChildIndex(enemies[i], 0);
					toRemove.push(enemies[i]);
				}
			}
			
			if (toRemove.length > 0)
			{
				for each (var en:Enemy in toRemove)
				{
					for (var j:int = 0; j < enemies.length; j++)
					{
						if (enemies[j] == en)
						{
							hud.removeEnemyLifeBar(enemies[j].warriorName);
							enemies.splice(j, 1);
							break;
						}
					}
				}
			}
			
			if (activeEnemies == 0)
			{
				endRound();
			}
			
			if (!player.active)
			{
				gameOver();
			}
		}
		
		public function endRound():void
		{
			SoundManager.getInstance().setMusicVolume(0.4);
			gameStarted = false;
			hud.displayWin();
			SoundManager.getInstance().playSound(GlobalSounds.SFX_PUBLIC);
			crowd.animate(1.0);
		}
		
		public function onPlayerWin(event:Event):void
		{
			currentRound++;
			startRound();
		}
		
		public function getPlayer():Warrior
		{
			return player;
		}
		
		public function gameOver():void
		{
			SoundManager.getInstance().stopMusic();
			SoundManager.getInstance().playSound(GlobalSounds.SFX_PUBLIC);
			crowd.animate(10.0);
			
			hud.showGameOver(currentRound);
		}
	}

}