package rogue 
{
	import flash.display.Sprite;
	import com.greensock.TweenLite;
	import com.greensock.easing.*;
	import flash.events.Event;
	import flash.filters.GlowFilter;
	/**
	 * ...
	 * @author Arthur Zeferino
	 */
	 
	public class Warrior extends GameSprite
	{
		protected var weapon:Weapon;
		protected var body:Sprite;
		protected var shield:Sprite;
		protected var armor:Sprite;
		
		protected var strength:Number;
		
		public var hurting:Boolean = false;		
		
		protected var attacking:Boolean = false;
		protected var holdingAttack:Boolean = false;
		
		public var maxHp:Number = 300;
		public var hp:Number = 300;
		
		protected var sparks:Sparks;
		
		public var active:Boolean = true;	
		
		public var controlLost:Boolean = false;
		
		public function Warrior() 
		{	
			//initialize();
			
			//this.body.filters = [new GlowFilter(0x0000ff, 1, 3, 3, 2)];
		}
		
		public function initialize(weaponType:int, hat:int, color:uint):void
		{
			body = new Sprite();
			body.graphics.lineStyle(2, color);
			Hats.drawHat(hat, body.graphics);
			body.x = -20;
			body.y = -20;
			this.addChild(body);
			
			this.collider = body;
			
			weapon = Weapon.newWeapon(weaponType);
			this.addChild(weapon);
			
			weapon.addEventListener(Weapon.EVENT_ATTACK_FINISH, onAttackFinish);
			
			shield = new Sprite();
			this.addChild(shield);
			shield.x = -10;
			shield.y = -25;
			
			shield.graphics.lineStyle(3, 0xffffff);
			shield.graphics.lineTo(25, 0);
			shield.rotation = 10;		
			
			sparks = new Sparks();
			this.addChild(sparks);
		}
		
		public function prepareAttack():void
		{
			if (!holdingAttack)
			{
				holdingAttack = true;
				strength = 0;
				weapon.prepare();
			}
			else
			{
				if (strength < 70)
				{
					strength+=10;
				}
				else
				{
					if (strength == 71)
						strength -= 1;
					else
						strength += 1;
				}
				
				weapon.charge(strength);
			}
		}
		
		/*Attack Mechanics*/
		public function attack():void
		{
			weapon.attack(strength);
			holdingAttack = false;
			attacking = true;
		}
		
		protected function onAttackFinish(event:Event):void
		{
			strength = 0;
			attacking = false;
			trace("attack finished");
		}
		
		public function isAttacking():Boolean
		{
			return attacking;
		}
		
		public function isHoldingAttacking():Boolean
		{
			return holdingAttack;
		}
		
		/*Shield Mechanics*/
		public function shieldUp():void
		{
			shield.x = 25;
			shield.y = -5;
			shield.rotation = 100;
		}
		
		public function shieldDown():void
		{
			shield.x = -10;
			shield.y = -25;
			shield.rotation = 10;
		}		
		
		public function isDefending():Boolean
		{
			return shield.x > 20;
		}
		
		public function get weaponSprite():Sprite
		{
			return weapon;
		}
		
		public function get shieldSprite():Sprite
		{
			return shield;
		}
		
		public function updateRotation(mX:Number, mY:Number):void
		{
			this.rotation = (Math.atan2(mY-y, mX-x) * 180 / Math.PI) - 30;
		}
		
		
		public function getStrength():Number
		{
			return strength;
		}
		
		
		public function hurt(plStrength:Number):void
		{
			if (!hurting)
			{
				hurting = true;
				this.x -= Math.cos(this.rotation * Math.PI / 180) * 10 - plStrength / 10;
				this.y -= Math.sin(this.rotation * Math.PI / 180) * 10 - plStrength / 10;
			
				hp -= plStrength;
			
				if (hp < 0)
				{
					die();
				}
				else
				{
					this.alpha = 0.5;
					TweenLite.delayedCall(0.3, heal);
				}
				
				sparks.visible = true;
				sparks.x = 20;
				sparks.y = 0;
				sparks.rotation = this.rotation;
				
				sparks.playAnim(0xff0000);
				
				SoundManager.getInstance().playHurtSound();
				
				TweenLite.to(sparks, 0.5, { alpha:0.0, x:30, onComplete:clearSparks } );				
			}
		}	
		
		public function heal():void
		{
			hurting = false;
			this.alpha = 1.0;
		}
		
		protected function die():void
		{
			this.active = false;
			TweenLite.to(this.body, 0.8, {x:-30, alpha:0.2, onComplete:removeBody});
		}
		
		protected function removeBody():void
		{
			body.visible = false;
		}
		
		public function reactDefense(plStrength:Number):void
		{
			this.x -= Math.cos(this.rotation * Math.PI / 180) * 20 - plStrength / 10;
			this.y -= Math.sin(this.rotation * Math.PI / 180) * 20 - plStrength / 10;
			
			sparks.visible = true;
			sparks.x = shield.x + shield.width/2;
			sparks.y = shield.y + shield.height/2;
			sparks.rotation = shield.rotation - 90;
			
			sparks.playAnim(0xffffff);
			
			SoundManager.getInstance().playSound(GlobalSounds.SFX_SHIELD);
			
			TweenLite.to(sparks, 0.2, { alpha:0.0, onComplete:clearSparks } );
		}			
		
		public function openDefense():void
		{
			shield.x = -15;
			shield.y = -30;
			shield.rotation = -20;
			
			weapon.loseControl();
			
			controlLost = true;		
			
			TweenLite.to(shield, 1, { x: -10, y: -25, rotation:10, onComplete:getControl});
		}
		
		protected function getControl():void
		{
			controlLost = false;
		}
		
		protected function clearSparks():void
		{
			sparks.clear();
			sparks.alpha = 1.0;
			sparks.visible = false;
		}
		
		override public function update():void
		{
		}
	}

}